/* cxscconf.h.  Generated from cxscconf.h.in by configure.  */
/* cxscconf.h.in.  Generated from configure.ac by autoheader.  */


#ifndef _CXSCCONF_H_INCLUDED
#define _CXSCCONF_H_INCLUDED


/* Use x86 ASM code if ROUND_ASM */
#define CXSC_x86 0

/* Use amd64 ASM code if ROUND_ASM */
#define CXSC_x86_64 0

/* Linux-ALPHA system */
#define DEC_ALPHA_C 0

/* 64-bit longs */
#define GNU_X86_64 0

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Compiling on a sane platform with a sane compiler */
#define IBM_LINUX_C 1

/* Compiling with AIX compiler */
#define IBM_RS_6000_C 0

/* Define to the sub-directory where libtool stores uninstalled libraries. */
#define LT_OBJDIR ".libs/"

/* Name of package */
#define PACKAGE "cxsc"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME "cxsc"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "cxsc 2.5.3"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "cxsc"

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION "2.5.3"

/* Employ ASM */
#define ROUND_ASM 0

/* */
#define ROUND_C96_QUICK 0

/* */
#define ROUND_C96_SAVE 0

/* */
#define ROUND_C99_QUICK 0

/* */
#define ROUND_C99_SAVE 1

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Compiling with Sun Forte (Solaris) */
#define SUN4_FORTE 0

/* Compiling with GCC under Solaris */
#define SUN4_GNU_C 0

/* Compiling with Sun Forte >= 5.7 (Solaris) */
#define SUN_STUDIO_10 0

/* Version number of package */
#define VERSION "2.5.3"

#endif
